import numpy as np
import pytest
import pylightcurve as plc

from pylightcurve.spacetime.angles import _is_angle, _request_angle


def test_angles():

    for test_angle in [plc.Degrees(54.61),
                       plc.Degrees('54.61'),
                       plc.Degrees(54, 36, 36.0),
                       plc.Degrees('54:36:36.0'),
                       plc.Hours(54.61 / 15.0),
                       plc.Hours(54 / 15.0, 36.0 / 15.0, 36.0 / 15.0),
                       plc.Rad(54.61 * np.pi / 180.0)]:

        assert round(test_angle.arcseconds, 1) == 196596.0
        assert [round(ff, 1) for ff in test_angle.dms().list] == [54, 36, 36]
        assert round(test_angle.deg(), 2) == 54.61
        assert round(test_angle.deg_coord(), 2) == 54.61
        assert [round(ff, 1) for ff in test_angle.hms().list] == [3, 38, 26.4]
        assert round(test_angle.hours(), 4) == 3.6407
        assert round(test_angle.rad(), 10) == 0.9531243045
        assert round(test_angle.sin(), 10) == 0.8152288870
        assert round(test_angle.cos(), 10) == 0.5791388969
        assert round(test_angle.tan(), 10) == 1.4076569392
        assert round(plc.arcsin(test_angle.sin()).deg(), 2) == 54.61
        assert round(plc.arccos(test_angle.cos()).deg(), 2) == 54.61
        assert round(plc.arctan(test_angle.tan()).deg(), 2) == 54.61
        assert _is_angle(test_angle) is True
        _request_angle(test_angle)

    for test_angle in [plc.Degrees(-54.61),
                       plc.Degrees('-54.61'),
                       plc.Degrees(-54, -36, -36.0),
                       plc.Degrees('-54:36:36.0'),
                       plc.Hours(-54.61 / 15.0),
                       plc.Hours(-54 / 15.0, -36.0 / 15.0, -36.0 / 15.0),
                       plc.Rad(-54.61 * np.pi / 180.0)]:

        assert round(test_angle.arcseconds, 1) == 1099404.0
        assert [round(ff, 1) for ff in test_angle.dms().list] == [305, 23, 24.0]
        assert round(test_angle.deg(), 2) == 305.39
        assert round(test_angle.deg_coord(), 2) == -54.61
        assert [round(ff, 1) for ff in test_angle.hms().list] == [20, 21, 33.6]
        assert round(test_angle.hours(), 4) == 20.3593
        assert round(test_angle.rad(), 10) == 5.3300610027
        assert round(test_angle.sin(), 10) == -0.8152288870
        assert round(test_angle.cos(), 10) == 0.5791388969
        assert round(test_angle.tan(), 10) == -1.4076569392
        assert _is_angle(test_angle) is True
        _request_angle(test_angle)

    a1, a2 = plc.Degrees(55.0), plc.Degrees(5.0)
    assert (a1 + a2).deg() == 60.0
    assert (a1 - a2).deg() == 50.0
    assert (a1 * 2).deg() == 110.0
    assert (2 * a1).deg() == 110.0
    assert (a1 / 2).deg() == 27.5

    assert plc.Degrees(5.0).deg_coord() == 5.0
    assert plc.Degrees(5.0).dms_coord() == '+05:00:00.0'
    assert plc.Degrees(95.0).deg_coord() == 85.0
    assert plc.Degrees(95.0).dms_coord() == '+85:00:00.0'
    assert plc.Degrees(185.0).deg_coord() == -5.0
    assert plc.Degrees(185.0).dms_coord() == '-05:00:00.0'
    assert plc.Degrees(275.0).deg_coord() == -85.0
    assert plc.Degrees(275.0).dms_coord() == '-85:00:00.0'

    assert plc.pi.rad() == np.pi

    print(test_angle.__repr__, test_angle.hms().__repr__, test_angle.dms().__repr__)

    with pytest.raises(plc.PyLCInputError):
        plc.Degrees('a')
    with pytest.raises(plc.PyLCInputError):
        plc.Degrees('35:00:00', 10)
    with pytest.raises(plc.PyLCInputError):
        plc.Degrees(['a'])
    with pytest.raises(plc.PyLCInputError):
        plc.Degrees(35, -10)
    with pytest.raises(plc.PyLCInputError):
        plc.Rad('a')
    with pytest.raises(plc.PyLCError):
        a1 * a2
    with pytest.raises(plc.PyLCError):
        a1 / a2
    with pytest.raises(plc.PyLCError):
        _request_angle('a')

    assert _is_angle('a') is False



import os
import pytest
import numpy as np
import pylightcurve as plc


def line(x_array, model_slope, model_constant, model_test):
    return model_slope * x_array + model_constant


def test_emcee():

    __location__ = os.path.abspath(os.path.dirname(__file__))

    data_x = np.arange(0, 10, 0.1)
    data_z = np.ones_like(data_x) * 0.1
    data_y = line(data_x, 1.3, 2.3, 0.0) + np.random.normal(0, data_z)

    def test_function():
        pass

    test_fit = plc.EmceeFitting(data_x, data_y, data_z, line, [1.5, 1.0, 0.0], [-5.0, -5.0, np.nan], [5.0, 5.0, np.nan],
                                200, 100000, 10000,
                                data_x_name='time', data_y_name='flux', data_x_print_name='Time',
                                data_y_print_name='Relative Flux',
                                parameters_names=['s', 'c', 't'], parameters_print_names=['slope', 'constant', 'test'],
                                function_to_call=test_function)

    test_fit.run_mcmc()
    assert round(test_fit.results['parameters']['s']['value'], 1) == 1.3
    assert round(test_fit.results['parameters']['c']['value'], 1) == 2.3

    test_fit = plc.EmceeFitting(data_x, data_y, data_z, line, [1.5, 1.0, 0.0], [-5.0, -5.0, np.nan], [5.0, 5.0, np.nan],
                                200, 100000, 10000,
                                data_x_name='time', data_y_name='flux', data_x_print_name='Time',
                                data_y_print_name='Relative Flux',
                                parameters_names=['s', 'c', 't'], parameters_print_names=['slope', 'constant', 'test'],
                                function_to_call=test_function)

    with pytest.raises(plc.PyLCProcessError):
        test_fit.save_all(os.path.join(__location__, 'emcee_test_data_base.pickle'))
    with pytest.raises(plc.PyLCProcessError):
        test_fit.save_results(os.path.join(__location__, 'emcee_test_results.txt'))
    with pytest.raises(plc.PyLCProcessError):
        test_fit.plot_corner(os.path.join(__location__, 'emcee_test_correlations.pdf'))
    with pytest.raises(plc.PyLCProcessError):
        test_fit.plot_traces(os.path.join(__location__, 'emcee_test_traces.pdf'))
    with pytest.raises(plc.PyLCProcessError):
        test_fit.plot_fitting(os.path.join(__location__, 'emcee_test_fitting.pdf'))

    test_fit.run_mcmc()

    try:
        os.mkdir(os.path.join(__location__, 'test_emcee'))
    except:
        pass

    test_fit.save_all(os.path.join(__location__, 'test_emcee', 'emcee_test_data_base.pickle'))
    test_fit.save_results(os.path.join(__location__, 'test_emcee', 'emcee_test_results.txt'))
    test_fit.plot_fitting(os.path.join(__location__, 'test_emcee', 'emcee_test_fitting.pdf'))
    test_fit.plot_corner(os.path.join(__location__, 'test_emcee', 'emcee_test_correlations.pdf'))
    test_fit.plot_traces(os.path.join(__location__, 'test_emcee', 'emcee_test_traces.pdf'))


def test_values_to_print():

    assert plc.values_to_print(3.1234567, 0.12456, 0.4345) == ('3.12', '0.12', '0.43')
    assert plc.values_to_print(3.1234567, 0.4345, 0.12456) == ('3.12', '0.43', '0.12')
    assert plc.values_to_print(3.1234567, 0.456, 0.657) == ('3.12', '0.46', '0.66')
    assert plc.values_to_print(3.1234567, 0.0, 0.00) == ('3.12', '0.00', '0.00')
    assert plc.values_to_print(3.1234567, 3.3, 2.65789) == ('3.12', '3.30', '2.66')
    assert plc.values_to_print(3.1234567, 3.0, 2.0) == ('3.12', '3.00', '2.00')




import numpy as np
import pytest
import pylightcurve as plc
from pylightcurve.models.exoplanet_lc import _get_filter


def test_gaussian():

    data_x = np.arange(0, 10, 0.01)
    data_y = plc.gaussian(data_x, 100.0, 10.0, 5.0, 1.0)

    results = plc.fit_gaussian(data_x, data_y)

    assert len(results) == 2
    assert round(results[0][0]) == 100
    assert round(results[0][1]) == 10
    assert round(results[0][2]) == 5
    assert round(results[0][3]) == 1

    results = plc.fit_gaussian(data_x, data_y, positive=True)

    assert len(results) == 2
    assert round(results[0][0]) == 100
    assert round(results[0][1]) == 10
    assert round(results[0][2]) == 5
    assert round(results[0][3]) == 1

    results = plc.fit_gaussian(data_x, data_y, positive=True, sampled=True)

    assert len(results) == 2
    assert round(results[0][0]) == 10000
    assert round(results[0][1]) == 1000
    assert round(results[0][2]) == 5
    assert round(results[0][3]) == 1

    data_x = np.arange(0, 10, 0.01)
    data_y = np.arange(0, 10, 0.01)
    data_x, data_y = np.meshgrid(data_x, data_y)
    data_z = plc.two_d_gaussian(data_x, data_y, 100, 10, 5, 5, 1, 1, 0)

    results = plc.fit_two_d_gaussian(data_x, data_y, data_z)

    assert len(results) == 2
    assert round(results[0][0]) == 100
    assert round(results[0][1]) == 10
    assert round(results[0][2]) == 5
    assert round(results[0][3]) == 5
    assert round(results[0][4]) == 1
    assert round(results[0][5]) == 1

    results = plc.fit_two_d_gaussian(data_x, data_y, data_z, positive=True)

    assert len(results) == 2
    assert round(results[0][0]) == 100
    assert round(results[0][1]) == 10
    assert round(results[0][2]) == 5
    assert round(results[0][3]) == 5
    assert round(results[0][4]) == 1
    assert round(results[0][5]) == 1

    results = plc.fit_two_d_gaussian(data_x, data_y, data_z, symmetric=True)

    assert len(results) == 2
    assert round(results[0][0]) == 100
    assert round(results[0][1]) == 10
    assert round(results[0][2]) == 5
    assert round(results[0][3]) == 5
    assert round(results[0][4]) == 1

    results = plc.fit_two_d_gaussian(data_x, data_y, data_z, positive=True, symmetric=True)

    assert len(results) == 2
    assert round(results[0][0]) == 100
    assert round(results[0][1]) == 10
    assert round(results[0][2]) == 5
    assert round(results[0][3]) == 5
    assert round(results[0][4]) == 1

    results = plc.fit_two_d_gaussian(data_x, data_y, data_z, point_xy=(5, 5), sigma=None)

    assert len(results) == 2
    assert round(results[0][0]) == 100
    assert round(results[0][1]) == 10
    assert round(results[0][2]) == 5
    assert round(results[0][3]) == 5
    assert round(results[0][4]) == 1
    assert round(results[0][5]) == 1

    results = plc.fit_two_d_gaussian(data_x, data_y, data_z, sigma=1)

    assert len(results) == 2
    assert round(results[0][0]) == 100
    assert round(results[0][1]) == 10
    assert round(results[0][2]) == 5
    assert round(results[0][3]) == 5
    assert round(results[0][4]) == 1
    assert round(results[0][5]) == 1




import numpy as np
import pytest
import pylightcurve as plc
from pylightcurve.models.exoplanet_lc import _get_filter
from pylightcurve.__databases__ import plc_data


def test_exoplanet_lc():

    stellar_logg = 4.36
    stellar_temperature = 6065.0
    stellar_metallicity = 0.0
    rp_over_rs = 0.12086
    period = 3.5247486
    sma_over_rs = 8.76
    eccentricity = 0.0
    inclination = 86.71
    periastron = 0.0
    mid_time = 2452826.62928
    mid_time_format = 'BJD_TDB'
    time_array = np.arange(mid_time - 0.1, mid_time + 0.1, 0.001)
    eclipse_mid_time = plc.eclipse_mid_time(period, sma_over_rs, eccentricity, inclination, periastron, mid_time)
    eclipse_time_array = np.arange(eclipse_mid_time - 0.1, eclipse_mid_time + 0.1, 0.001)

    planet_orbit = plc.planet_orbit(period, sma_over_rs, eccentricity, inclination, periastron, mid_time, time_array)

    assert len(planet_orbit) == 3
    assert len(planet_orbit[0]) == len(time_array)
    assert len(planet_orbit[1]) == len(time_array)
    assert len(planet_orbit[2]) == len(time_array)
    assert min(planet_orbit[0]) > 8.6
    assert min(planet_orbit[1]) > -1.6
    assert min(planet_orbit[2]) > -0.6
    assert max(planet_orbit[0]) < 8.8
    assert max(planet_orbit[1]) < 1.6
    assert max(planet_orbit[2]) < -0.4

    planet_star_projected_distance = plc.planet_star_projected_distance(
        period, sma_over_rs, eccentricity, inclination, periastron, mid_time, time_array)

    assert min(planet_star_projected_distance) > 0.5
    assert max(planet_star_projected_distance) < 1.7

    planet_phase = plc.planet_phase(period, mid_time, time_array)

    assert min(planet_phase) > -0.1
    assert max(planet_phase) < 0.1

    for photometric_filter in plc_data.all_filters():

        for method in ['claret', 'sqrt', 'quad', 'linear']:

            for stellar_model in ['phoenix', 'atlas']:

                limb_darkening_coefficients = plc.exotethys(stellar_logg, stellar_temperature,
                                                            stellar_metallicity, photometric_filter,
                                                            method=method, stellar_model=stellar_model)

                assert len(limb_darkening_coefficients) == 4

                transit = plc.transit(limb_darkening_coefficients, rp_over_rs, period, sma_over_rs, eccentricity, inclination,
                                      periastron, mid_time, time_array, method=method)

                assert min(transit) > 0.98
                assert max(transit) == 1.0

                transit_integrated = plc.transit_integrated(limb_darkening_coefficients, rp_over_rs, period, sma_over_rs,
                                                            eccentricity, inclination, periastron, mid_time, time_array,
                                                            30, max_sub_exp_time=10, method=method, precision=3)

                assert min(transit_integrated) > 0.98
                assert max(transit_integrated) == 1.0

                assert round(plc.transit_duration(rp_over_rs, period, sma_over_rs, eccentricity, inclination,
                                                  periastron) * 24, 2) == 3.09

                assert round(plc.transit_depth(limb_darkening_coefficients, rp_over_rs, period, sma_over_rs, eccentricity,
                                               inclination, periastron), 2) <= 0.02

        fp_over_fs = plc.fp_over_fs(rp_over_rs, sma_over_rs, 0.2, 1.0, stellar_temperature, photometric_filter)

        eclipse = plc.eclipse(fp_over_fs, rp_over_rs, period, sma_over_rs, eccentricity, inclination, periastron,
                              eclipse_mid_time, eclipse_time_array)

        assert min(eclipse) > 0.99
        assert max(eclipse) == 1.0

        eclipse_integrated = plc.eclipse_integrated(fp_over_fs, rp_over_rs, period, sma_over_rs, eccentricity,
                                                    inclination, periastron, eclipse_mid_time, eclipse_time_array,
                                                    30, max_sub_exp_time=10)

        assert min(eclipse_integrated) > 0.998
        assert max(eclipse_integrated) == 1.0

        assert round(plc.eclipse_duration(rp_over_rs, period, sma_over_rs, eccentricity, inclination,
                                          periastron) * 24, 2) == 3.09

        assert round(plc.eclipse_depth(fp_over_fs, rp_over_rs, period, sma_over_rs, eccentricity,
                                       inclination, periastron), 5) <= 0.002

        planet_orbit = plc.planet_orbit(period, sma_over_rs, 0.1, inclination, 80.0, mid_time,
                                        time_array)

        assert len(planet_orbit) == 3
        assert len(planet_orbit[0]) == len(time_array)
        assert len(planet_orbit[1]) == len(time_array)
        assert len(planet_orbit[2]) == len(time_array)
        assert min(planet_orbit[0]) > 7.6
        assert min(planet_orbit[1]) > -1.8
        assert min(planet_orbit[2]) > -0.6
        assert max(planet_orbit[0]) < 8.8
        assert max(planet_orbit[1]) < 1.8
        assert max(planet_orbit[2]) < -0.4

        planet_orbit = plc.planet_orbit(period, sma_over_rs, 0.1, inclination, 130.0, mid_time,
                                        time_array)

        assert len(planet_orbit) == 3
        assert len(planet_orbit[0]) == len(time_array)
        assert len(planet_orbit[1]) == len(time_array)
        assert len(planet_orbit[2]) == len(time_array)
        assert min(planet_orbit[0]) > 7.6
        assert min(planet_orbit[1]) > -1.8
        assert min(planet_orbit[2]) > -0.6
        assert max(planet_orbit[0]) < 8.8
        assert max(planet_orbit[1]) < 1.8
        assert max(planet_orbit[2]) < -0.4

    with pytest.raises(plc.PyLCError):
        _get_filter('aa')

    with pytest.raises(plc.PyLCError):
        plc.exotethys(stellar_logg, stellar_temperature, stellar_metallicity, photometric_filter,
                      method='bla', stellar_model='phoenix')

    with pytest.raises(plc.PyLCError):
        plc.exotethys(stellar_logg, stellar_temperature, stellar_metallicity, photometric_filter,
                      method='claret', stellar_model='bla')

    with pytest.raises(plc.PyLCProcessError):
        plc.exotethys(1.0, 100000, 30, 'JOHNSON_V', method='claret', stellar_model='phoenix')

    assert len(plc.exotethys(stellar_logg, stellar_temperature, 0.1, 'JOHNSON_V', method='claret',
                             stellar_model='phoenix')) == 4



import pytest
import numpy as np
import pylightcurve as plc
import os
from astropy.io import fits as pf


def test_files():

    __location__ = os.path.abspath(os.path.dirname(__file__))

    xx = {'a': 1}
    plc.save_dict(xx, os.path.join(__location__, 'test.pickle'))
    xx = plc.open_dict(os.path.join(__location__, 'test.pickle'))
    os.remove(os.path.join(__location__, 'test.pickle'))
    yy = plc.copy_dict(xx)
    assert id(yy) != id(xx)
    del xx, yy

    xx = {'a': 1}
    plc.save_yaml(xx, os.path.join(__location__, 'test.yaml'))
    xx = plc.open_yaml(os.path.join(__location__, 'test.yaml'))
    os.remove(os.path.join(__location__, 'test.yaml'))
    yy = plc.copy_yaml(xx)
    assert id(yy) != id(xx)
    del xx, yy

    if os.path.isfile(os.path.join(__location__, 'test.fits')):
        os.remove(os.path.join(__location__, 'test.fits'))
    xx = pf.HDUList([pf.PrimaryHDU(), pf.ImageHDU(data=np.array([[1, 2], [1, 2]]), name='SCI'),
                     pf.ImageHDU(data=np.array([[1, 2], [1, 2]]), name='ERR')])

    print(plc.fits_sci(xx), plc.fits_err(xx), plc.fits_sci_err(xx))

    assert plc.fits_sci(xx)[0] == 1
    assert plc.fits_err(xx)[0] == 2
    assert plc.fits_sci_err(xx)[0][0] == 1
    assert plc.fits_sci_err(xx)[0][1] == 2

    xx[1].header.set('A?χ', 3)
    plc.save_fits(xx, os.path.join(__location__, 'test.fits'))
    xx = plc.open_fits(os.path.join(__location__, 'test.fits'))
    os.remove(os.path.join(__location__, 'test.fits'))
    yy = plc.copy_fits(xx)
    assert id(yy) != id(xx)
    del xx, yy

    plc.download('https://raw.githubusercontent.com/ucl-exoplanets/pylightcurve/master/README.md',
                 os.path.join(__location__, 'test.txt'))
    os.remove(os.path.join(__location__, 'test.txt'))

    plc.download('https://raw.githubusercontent.com/ucl-exoplanets/pylightcurve/master/README.mx',
                 os.path.join(__location__, 'test.txt'))






import pytest
import numpy as np
import pylightcurve as plc


def test_distribution_one_d():

    test_mean = 10.0
    test_std = 1.0
    distr = np.append(np.random.normal(test_mean, test_std, 100000), 1000000)

    a = plc.one_d_distribution(distr, mad_filter=5)
    assert round(a[0][np.argmax(a[1])]) == test_mean
    assert round(5 * (a[0][1]-a[0][0])) == test_std

    distr = np.random.normal(test_mean, test_std, 100000)

    a = plc.one_d_distribution(distr)
    assert round(a[0][np.argmax(a[1])]) == test_mean
    assert round(5 * (a[0][1]-a[0][0])) == test_std

    b = plc.one_d_distribution(distr, step=10)
    assert round(b[0][np.argmax(b[1])]) == test_mean
    assert round(10 * (b[0][1]-b[0][0])) == test_std

    c = plc.one_d_distribution(distr, abs_step=1.6, min_value=5.0, max_value=15.0)
    assert round(c[0][np.argmax(c[1])], 1) == 10.6
    assert round(c[0][1]-c[0][0], 1) == 1.6
    assert round(c[0][0], 1) == 5.8
    assert round(c[0][-1], 1) == 15.4

    c = plc.one_d_distribution(distr, abs_step=1.6, min_value=5.0)
    assert round(c[0][0], 1) == 5.8

    c = plc.one_d_distribution(distr, abs_step=1.6, max_value=15.0)
    assert round(c[0][-1], 1) < 16.6

    a = plc.one_d_distribution(distr, confidence_interval=0.68)
    assert len(a) == 5
    assert round(a[0][np.argmax(a[1])]) == test_mean
    assert round(5 * (a[0][1] - a[0][0])) == test_std
    assert round(a[2]) == test_mean
    assert round(a[3]) == test_std
    assert round(a[4]) == test_std
    a = plc.one_d_distribution(distr, gaussian_fit=True)
    assert len(a) == 5
    assert round(a[0][np.argmax(a[1])]) == test_mean
    assert round(5 * (a[0][1] - a[0][0])) == test_std
    assert round(a[2]) == test_mean
    assert round(a[3]) == test_std
    assert round(a[4]) == test_std

    with pytest.raises(plc.PyLCProcessError):
        plc.one_d_distribution(distr, confidence_interval=0.68, gaussian_fit=True)


def test_distribution_two_d():

    test_mean_1 = 10.0
    test_std_1 = 1.0
    test_mean_2 = 20.0
    test_std_2 = 2.0

    distr_1 = np.append(np.random.normal(test_mean_1, test_std_1, 100000), 100000)
    distr_2 = np.append(np.random.normal(test_mean_2, test_std_2, 100000), 100000)

    a = plc.two_d_distribution(distr_1, distr_2, mad_filter=5)
    assert abs(a[0].flatten()[np.argmax(a[2])] - test_mean_1) < test_std_1
    assert abs(a[1].flatten()[np.argmax(a[2])] - test_mean_2) < test_std_2
    assert round(5 * (a[0][0][1]-a[0][0][0]), 1) == 0.7
    assert round(5 * (a[1][1][0]-a[1][0][0]), 1) in [1.3, 1.4]

    distr_1 = np.random.normal(test_mean_1, test_std_1, 100000)
    distr_2 = np.random.normal(test_mean_2, test_std_2, 100000)

    a = plc.two_d_distribution(distr_1, distr_2)
    assert abs(a[0].flatten()[np.argmax(a[2])] - test_mean_1) < test_std_1
    assert abs(a[1].flatten()[np.argmax(a[2])] - test_mean_2) < test_std_2
    assert round(5 * (a[0][0][1]-a[0][0][0]), 1) == 0.7
    assert round(5 * (a[1][1][0]-a[1][0][0]), 1) in [1.3, 1.4]

    c = plc.two_d_distribution(distr_1, distr_2, step=1, min_value_x=5.0, min_value_y=15.0)
    assert round(np.min(c[0]), 1) >= 5.0
    assert round(np.min(c[1]), 1) >= 15.0

    c = plc.two_d_distribution(distr_1, distr_2, step=1, max_value_x=15.0, max_value_y=25.0)
    assert round(np.max(c[0]), 1) <= 15.5
    assert round(np.max(c[1]), 1) <= 25.7

    c = plc.two_d_distribution(distr_1, distr_2, step=1, min_value_x=5.0, max_value_x=15.0, min_value_y=15.0,
                               max_value_y=25.0)
    assert round(np.min(c[0]), 1) >= 5.0
    assert round(np.max(c[0]), 1) <= 15.6
    assert round(np.min(c[1]), 1) >= 15.0
    assert round(np.max(c[1]), 1) <= 25.7

    a = plc.two_d_distribution(distr_1, distr_2, step=10)
    assert abs(a[0].flatten()[np.argmax(a[2])] - test_mean_1) < test_std_1
    assert abs(a[1].flatten()[np.argmax(a[2])] - test_mean_2) < test_std_2
    assert round(10 * (a[0][0][1]-a[0][0][0]), 1) == 0.7
    assert round(10 * (a[1][1][0]-a[1][0][0]), 1) in [1.3, 1.4]

    a = plc.two_d_distribution(distr_1, distr_2, abs_step_x=0.1, abs_step_y=0.1)
    assert abs(a[0].flatten()[np.argmax(a[2])] - test_mean_1) < test_std_1
    assert abs(a[1].flatten()[np.argmax(a[2])] - test_mean_2) < test_std_2
    assert round(a[0][0][1]-a[0][0][0], 1) == 0.1
    assert round(a[1][1][0]-a[1][0][0], 1) == 0.1

    a = plc.two_d_distribution(distr_1, distr_2, confidence_interval=0.68)
    assert len(a) == 9
    assert abs(a[0].flatten()[np.argmax(a[2])] - test_mean_1) < test_std_1
    assert abs(a[1].flatten()[np.argmax(a[2])] - test_mean_2) < test_std_2
    assert round(5 * (a[0][0][1]-a[0][0][0]), 1) == 0.7
    assert round(5 * (a[1][1][0]-a[1][0][0]), 1) in [1.3, 1.4]
    assert round(a[3]) == test_mean_1
    assert round(a[4]) == test_std_1
    assert round(a[5]) == test_std_1
    assert round(a[6]) == test_mean_2
    assert round(a[7]) == test_std_2
    assert round(a[8]) == test_std_2

    a = plc.two_d_distribution(distr_1, distr_2, gaussian_fit=True)
    assert len(a) == 9
    assert abs(a[0].flatten()[np.argmax(a[2])] - test_mean_1) < test_std_1
    assert abs(a[1].flatten()[np.argmax(a[2])] - test_mean_2) < test_std_2
    assert round(5 * (a[0][0][1]-a[0][0][0]), 1) == 0.7
    assert round(5 * (a[1][1][0]-a[1][0][0]), 1) in [1.3, 1.4]
    assert round(a[3]) == test_mean_1
    assert round(a[4]) == test_std_1
    assert round(a[5]) == test_std_1
    assert round(a[6]) == test_mean_2
    assert round(a[7]) == test_std_2
    assert round(a[8]) == test_std_2

    with pytest.raises(plc.PyLCProcessError):
        plc.two_d_distribution(distr_1, distr_2, confidence_interval=0.68, gaussian_fit=True)



import pytest
import pylightcurve as plc

from pylightcurve.spacetime.targets import _is_target, _request_target


def test_targets():

    test_fixedstar = plc.FixedTarget(plc.Hours(3, 15), plc.Degrees(-45.7))
    assert test_fixedstar.coord == '03:15:00.0 -45:42:00.0'

    ra, dec = plc.Hours('03 15 00'), plc.Degrees('45 42 00')
    test_fixedstar = plc.FixedTarget(ra, dec)
    assert test_fixedstar.coord == '03:15:00.0 +45:42:00.0'

    print(test_fixedstar.distance_on_sphere(test_fixedstar).deg())

    assert test_fixedstar.distance_on_sphere(test_fixedstar).deg() == 0

    _request_target(test_fixedstar)
    assert _is_target(test_fixedstar) is True
    print(test_fixedstar.__repr__)

    assert round(test_fixedstar.convert_to_bjd_tdb(2458485.0, 'JD_UTC'), 7) == 2458485.0046340
    assert round(test_fixedstar.convert_to_bjd_tdb(58484.5, 'MJD_UTC'), 7) == 2458485.0046340
    assert round(test_fixedstar.convert_to_bjd_tdb(2458485.0046340, 'BJD_TDB'), 7) == 2458485.0046340
    assert round(test_fixedstar.convert_to_bjd_tdb(2458485.0038333, 'BJD_UTC'), 7) == 2458485.0046340
    assert round(test_fixedstar.convert_to_bjd_tdb(2458485.0046033, 'HJD_TDB'), 7) == 2458485.0046340
    assert round(test_fixedstar.convert_to_bjd_tdb(2458485.00380255, 'HJD_UTC'), 7) == 2458485.0046340

    assert round(test_fixedstar.convert_to_bjd_tdb([2458485.0], 'JD_UTC')[0], 7) == 2458485.0046340
    assert round(test_fixedstar.convert_to_bjd_tdb([58484.5], 'MJD_UTC')[0], 7) == 2458485.0046340
    assert round(test_fixedstar.convert_to_bjd_tdb([2458485.0046340], 'BJD_TDB')[0], 7) == 2458485.0046340
    assert round(test_fixedstar.convert_to_bjd_tdb([2458485.0038333], 'BJD_UTC')[0], 7) == 2458485.0046340
    assert round(test_fixedstar.convert_to_bjd_tdb([2458485.0046033], 'HJD_TDB')[0], 7) == 2458485.0046340
    assert round(test_fixedstar.convert_to_bjd_tdb([2458485.00380255], 'HJD_UTC')[0], 7) == 2458485.0046340

    assert round(test_fixedstar.convert_to_jd(2458485.0, 'JD_UTC'), 7) == 2458485.0
    assert round(test_fixedstar.convert_to_jd(58484.5, 'MJD_UTC'), 7) == 2458485.0
    assert round(test_fixedstar.convert_to_jd(2458485.0046340, 'BJD_TDB'), 7) == 2458485.0
    assert round(test_fixedstar.convert_to_jd(2458485.0038333, 'BJD_UTC'), 7) == 2458485.0
    assert round(test_fixedstar.convert_to_jd(2458485.0046033, 'HJD_TDB'), 7) == 2458485.0
    assert round(test_fixedstar.convert_to_jd(2458485.00380255, 'HJD_UTC'), 7) == 2458485.0

    assert round(test_fixedstar.convert_to_jd([2458485.0], 'JD_UTC')[0], 7) == 2458485.0
    assert round(test_fixedstar.convert_to_jd([58484.5], 'MJD_UTC')[0], 7) == 2458485.0
    assert round(test_fixedstar.convert_to_jd([2458485.0046340], 'BJD_TDB')[0], 7) == 2458485.0
    assert round(test_fixedstar.convert_to_jd([2458485.0038333], 'BJD_UTC')[0], 7) == 2458485.0
    assert round(test_fixedstar.convert_to_jd([2458485.0046033], 'HJD_TDB')[0], 7) == 2458485.0
    assert round(test_fixedstar.convert_to_jd([2458485.00380255], 'HJD_UTC')[0], 7) == 2458485.0

    assert round(test_fixedstar.convert_to_mjd(2458485.0, 'JD_UTC'), 7) == 58484.5
    assert round(test_fixedstar.convert_to_mjd(58484.5, 'MJD_UTC'), 7) == 58484.5
    assert round(test_fixedstar.convert_to_mjd(2458485.0046340, 'BJD_TDB'), 7) == 58484.5
    assert round(test_fixedstar.convert_to_mjd(2458485.0038333, 'BJD_UTC'), 7) == 58484.5
    assert round(test_fixedstar.convert_to_mjd(2458485.0046033, 'HJD_TDB'), 7) == 58484.5
    assert round(test_fixedstar.convert_to_mjd(2458485.00380255, 'HJD_UTC'), 7) == 58484.5

    assert round(test_fixedstar.convert_to_mjd([2458485.0], 'JD_UTC')[0], 7) == 58484.5
    assert round(test_fixedstar.convert_to_mjd([58484.5], 'MJD_UTC')[0], 7) == 58484.5
    assert round(test_fixedstar.convert_to_mjd([2458485.0046340], 'BJD_TDB')[0], 7) == 58484.5
    assert round(test_fixedstar.convert_to_mjd([2458485.0038333], 'BJD_UTC')[0], 7) == 58484.5
    assert round(test_fixedstar.convert_to_mjd([2458485.0046033], 'HJD_TDB')[0], 7) == 58484.5
    assert round(test_fixedstar.convert_to_mjd([2458485.00380255], 'HJD_UTC')[0], 7) == 58484.5

    with pytest.raises(plc.PyLCInputError):
        test_fixedstar.convert_to_bjd_tdb(2458485.0, 'aaa')
    with pytest.raises(plc.PyLCInputError):
        test_fixedstar.convert_to_bjd_tdb([2458485.0], 'aaa')
    with pytest.raises(plc.PyLCInputError):
        test_fixedstar.convert_to_bjd_tdb('a', 'JD_UTC')

    with pytest.raises(plc.PyLCInputError):
        test_fixedstar.convert_to_jd(2458485.0, 'aaa')
    with pytest.raises(plc.PyLCInputError):
        test_fixedstar.convert_to_jd([2458485.0], 'aaa')
    with pytest.raises(plc.PyLCInputError):
        test_fixedstar.convert_to_jd('a', 'JD_UTC')

    with pytest.raises(plc.PyLCInputError):
        test_fixedstar.convert_to_mjd(2458485.0, 'aaa')
    with pytest.raises(plc.PyLCInputError):
        test_fixedstar.convert_to_mjd([2458485.0], 'aaa')
    with pytest.raises(plc.PyLCInputError):
        test_fixedstar.convert_to_mjd('a', 'JD_UTC')

    with pytest.raises(plc.PyLCInputError):
        plc.FixedTarget('a', plc.Degrees(23.5))
    with pytest.raises(plc.PyLCInputError):
        plc.FixedTarget(plc.Degrees(40.4), plc.Degrees(170))
    with pytest.raises(plc.PyLCInputError):
        plc.FixedTarget(plc.Degrees(40.4), 'a')

    with pytest.raises(plc.PyLCInputError):
        _request_target('a')

    assert _is_target('a') is False





import pytest
import pylightcurve as plc
import os


def test_find_stars():
    __location__ = os.path.abspath(os.path.dirname(__file__))

    fits = plc.open_fits(os.path.join(__location__, 'out_2016_08_03_19_08_10_qatar1b-001.fit'))

    single = plc.find_single_star(fits[1].data, 685, 435)
    assert len(single) == 8

    single = plc.find_single_star(fits[1].data, 0, 0)
    assert single is None




import os
import numpy as np
import pytest
import shutil
import pylightcurve as plc
from pylightcurve.__databases__ import plc_data


def test_exoplanet():

    ra = 330.795
    dec = 18.884
    stellar_logg = 4.36
    stellar_temperature = 6065.0
    stellar_metallicity = 0.0
    rp_over_rs = 0.12086
    period = 3.5247486
    sma_over_rs = 8.76
    eccentricity = 0.0
    inclination = 86.71
    periastron = 0.0
    mid_time = 2452826.62928
    mid_time_format = 'BJD_TDB'
    time_array = np.arange(mid_time - 0.1, mid_time + 0.1, 0.001)

    planet = plc.Planet('HD209458b', plc.Degrees(ra), plc.Degrees(dec),
                        stellar_logg, stellar_temperature, stellar_metallicity,
                        rp_over_rs, period, sma_over_rs, eccentricity, plc.Degrees(inclination),
                        plc.Degrees(periastron), mid_time, mid_time_format)

    planet = plc.Planet('HD209458b', ra, dec,
                        stellar_logg, stellar_temperature, stellar_metallicity,
                        rp_over_rs, period, sma_over_rs, eccentricity, inclination,
                        periastron, mid_time, mid_time_format)

    eclipse_mid_time = planet.eclipse_mid_time
    eclipse_time_array = np.arange(eclipse_mid_time - 0.1, eclipse_mid_time + 0.1, 0.001)

    planet_orbit = planet.planet_orbit(time_array, 'BJD_TDB')

    assert len(planet_orbit) == 3
    assert len(planet_orbit[0]) == len(time_array)
    assert len(planet_orbit[1]) == len(time_array)
    assert len(planet_orbit[2]) == len(time_array)
    assert min(planet_orbit[0]) > 8.6
    assert min(planet_orbit[1]) > -1.6
    assert min(planet_orbit[2]) > -0.6
    assert max(planet_orbit[0]) < 8.8
    assert max(planet_orbit[1]) < 1.6
    assert max(planet_orbit[2]) < -0.4

    planet_star_projected_distance = planet.planet_star_projected_distance(time_array, 'BJD_TDB')

    assert min(planet_star_projected_distance) > 0.5
    assert max(planet_star_projected_distance) < 1.7

    planet_phase = planet.planet_phase(time_array, 'BJD_TDB')

    assert min(planet_phase) > -0.1
    assert max(planet_phase) < 0.1

    for photometric_filter in plc_data.all_filters():

        print(photometric_filter)

        transit = planet.transit(time_array, 'BJD_TDB', photometric_filter)

        assert min(transit) > 0.98
        assert max(transit) == 1.0

        transit_integrated = planet.transit_integrated(time_array, 'BJD_TDB', 30, 'mid', photometric_filter)
        transit_integrated = planet.transit_integrated(time_array, 'BJD_TDB', 30, 'start', photometric_filter)
        transit_integrated = planet.transit_integrated(time_array, 'BJD_TDB', 30, 'end', photometric_filter)

        with pytest.raises(plc.PyLCInputError):
            transit_integrated = planet.transit_integrated(time_array, 'BJD_TDB', 30, 'bla', photometric_filter)

        assert min(transit_integrated) > 0.98
        assert max(transit_integrated) == 1.0

        assert round(planet.transit_duration(photometric_filter) * 24, 2) == 3.09

        assert round(planet.transit_depth(photometric_filter), 2) <= 0.02

        eclipse = planet.eclipse(eclipse_time_array, 'BJD_TDB', photometric_filter)

        assert min(eclipse) > 0.99
        assert max(eclipse) == 1.0

        eclipse_integrated = planet.eclipse_integrated(eclipse_time_array, 'BJD_TDB', 30, 'mid', photometric_filter)
        eclipse_integrated = planet.eclipse_integrated(eclipse_time_array, 'BJD_TDB', 30, 'start', photometric_filter)
        eclipse_integrated = planet.eclipse_integrated(eclipse_time_array, 'BJD_TDB', 30, 'end', photometric_filter)

        with pytest.raises(plc.PyLCInputError):
            eclipse_integrated = planet.eclipse_integrated(eclipse_time_array, 'BJD_TDB', 30, 'bla', photometric_filter)

        assert min(eclipse_integrated) > 0.998
        assert max(eclipse_integrated) == 1.0

        assert round(planet.eclipse_duration(photometric_filter) * 24, 2) == 3.09

        assert round(planet.eclipse_depth(photometric_filter), 5) <= 0.002

    with pytest.raises(plc.PyLCInputError):
        planet.filter('test')

    planet.add_filter('test', 0.1, 1, 1, 1, 1, 0.001)

    __location__ = os.path.abspath(os.path.dirname(__file__))

    planet.observations = {}

    data_time = time_array
    data_flux = planet.transit_integrated(time_array, 'BJD_TDB', 60, 'mid', 'COUSINS_R')

    planet.add_observation(data_time, 'BJD_TDB', 60, 'start',
                           data_flux + np.random.normal(0, 0.001, len(data_time)),
                           np.ones_like(data_flux) * 0.001, 'flux', 'COUSINS_R')

    planet.add_observation(data_time, 'BJD_TDB', 60, 'end',
                           data_flux + np.random.normal(0, 0.001, len(data_time)),
                           np.ones_like(data_flux) * 0.001, 'flux', 'COUSINS_R')

    planet.add_observation(data_time, 'BJD_TDB', 60, 'end',
                           data_flux + np.random.normal(0, 0.001, len(data_time)),
                           np.ones_like(data_flux) * 0.001, 'mag', 'COUSINS_R')

    with pytest.raises(plc.PyLCInputError):
        planet.add_observation(data_time, 'BJD_TDB', 60, 'bla',
                               data_flux + np.random.normal(0, 0.001, len(data_time)),
                               np.ones_like(data_flux) * 0.001, 'flux', 'COUSINS_R')

    with pytest.raises(plc.PyLCInputError):
        planet.add_observation(data_time, 'BJD_TDB', 60, 'mid',
                               data_flux + np.random.normal(0, 0.001, len(data_time)),
                               np.ones_like(data_flux) * 0.001, 'bla', 'COUSINS_R')

    planet.observations = {}

    data_time = time_array
    data_flux = planet.transit_integrated(time_array, 'BJD_TDB', 60, 'mid', 'COUSINS_R')

    planet.add_observation(data_time, 'BJD_TDB', 60, 'mid',
        data_flux + np.random.normal(0, 0.001, len(data_time)),
        np.ones_like(data_flux) * 0.001, 'flux', 'COUSINS_R')

    try:
        shutil.rmtree(os.path.join(__location__, 'test_transit'))
    except:
        pass

    planet.transit_fitting(os.path.join(__location__, 'test_transit'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=2,
                           fit_rp_over_rs=True, fit_individual_rp_over_rs=True,
                           fit_mid_time=True, fit_individual_times=True)

    planet.transit_fitting(os.path.join(__location__, 'test_transit'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=2,
                           fit_rp_over_rs=False, fit_individual_rp_over_rs=True,
                           fit_mid_time=False, fit_individual_times=True)

    planet.transit_fitting(os.path.join(__location__, 'test_transit'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=1,
                           fit_rp_over_rs=True, fit_individual_rp_over_rs=False,
                           fit_mid_time=True, fit_individual_times=False)

    planet.transit_fitting(os.path.join(__location__, 'test_transit'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=1,
                           fit_rp_over_rs=False, fit_individual_rp_over_rs=False,
                           fit_mid_time=False, fit_individual_times=False)

    planet.transit_fitting(os.path.join(__location__, 'test_transit'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=0,
                           fit_ldc1=True, fit_ldc2=True, fit_ldc3=True, fit_ldc4=True,
                           fit_sma_over_rs=True, fit_inclination=True,
                           fit_mid_time=True, fit_individual_times=True
                           )

    planet.transit_fitting(os.path.join(__location__, 'test_transit'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=0,
                           fit_period=True, fit_mid_time=True, fit_individual_times=False
                           )

    with pytest.raises(plc.PyLCInputError):
        planet.transit_fitting(os.path.join(__location__, 'test_transit'),
                               iterations=1300, walkers=200, burn_in=300,
                               detrending_order=0,
                               fit_period=True, fit_mid_time=True, fit_individual_times=True
                               )

    planet.observations = {}

    data_time = eclipse_time_array
    data_flux = planet.eclipse_integrated(eclipse_time_array, 'BJD_TDB', 60, 'mid', 'COUSINS_R')

    planet.add_observation(data_time, 'BJD_TDB', 60, 'mid',
                           data_flux + np.random.normal(0, 0.00001, len(data_time)),
                           np.ones_like(data_flux) * 0.00001, 'flux', 'COUSINS_R')

    try:
        shutil.rmtree(os.path.join(__location__, 'test_eclipse'))
    except:
        pass

    planet.eclipse_fitting(os.path.join(__location__, 'test_eclipse'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=2,
                           fit_rp_over_rs=True, fit_individual_rp_over_rs=True,
                           fit_fp_over_fs=True, fit_individual_fp_over_fs=True,
                           fit_mid_time=True, fit_individual_times=True)

    planet.eclipse_fitting(os.path.join(__location__, 'test_eclipse'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=2,
                           fit_rp_over_rs=False, fit_individual_rp_over_rs=True,
                           fit_fp_over_fs=False, fit_individual_fp_over_fs=True,
                           fit_mid_time=False, fit_individual_times=True)

    planet.eclipse_fitting(os.path.join(__location__, 'test_eclipse'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=1,
                           fit_rp_over_rs=True, fit_individual_rp_over_rs=False,
                           fit_fp_over_fs=True, fit_individual_fp_over_fs=False,
                           fit_mid_time=True, fit_individual_times=False)

    planet.eclipse_fitting(os.path.join(__location__, 'test_eclipse'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=1,
                           fit_rp_over_rs=False, fit_individual_rp_over_rs=False,
                           fit_fp_over_fs=False, fit_individual_fp_over_fs=False,
                           fit_mid_time=False, fit_individual_times=False)

    planet.eclipse_fitting(os.path.join(__location__, 'test_eclipse'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=0,
                           fit_sma_over_rs=True, fit_inclination=True,
                           fit_mid_time=True, fit_individual_times=True
                           )

    planet.eclipse_fitting(os.path.join(__location__, 'test_eclipse'),
                           iterations=1300, walkers=200, burn_in=300,
                           detrending_order=0,
                           fit_period=True, fit_mid_time=True, fit_individual_times=False
                           )

    with pytest.raises(plc.PyLCInputError):
        planet.eclipse_fitting(os.path.join(__location__, 'test_eclipse'),
                               iterations=1300, walkers=200, burn_in=300,
                               detrending_order=0,
                               fit_period=True, fit_mid_time=True, fit_individual_times=True
                               )




import pytest
import pylightcurve as plc


def test_catalogues():

    planet1 = plc.get_planet('hatp7b')
    planet2 = plc.get_planet('HAT-P-7b')
    assert planet1.name == 'HAT-P-7b'
    assert planet2.name == 'HAT-P-7b'
    assert planet1.sma_over_rs == planet2.sma_over_rs

    planet = plc.get_planet('wasp77b')
    assert planet.name == 'WASP-77Ab'

    with pytest.raises(plc.PyLCInputError):
        planet = plc.get_planet('aaaaaaaaaa')

    assert len(plc.get_all_planets()) > 0

    planet = plc.locate_planet(plc.Degrees(330.795), plc.Degrees(18.884))
    assert planet.name == 'HD209458b'

    planet = plc.locate_planet(330.795, 18.884)
    assert planet.name == 'HD209458b'

    with pytest.raises(plc.PyLCLibraryError):
        planet = plc.locate_planet(plc.Degrees(330.795), plc.Degrees(17.884))
